@@ -1,5 +1,5 @@
 /*
-  Copyright 2022 Northern.tech AS
+  Copyright 2024 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -26,6 +26,7 @@
 #define CF_NET_VERSION "0.1.2"
 
 #include <platform.h>
+#include <getopt.h>
 #include <libgen.h>
 #include <client_code.h>        // ServerConnection
 #include <logging.h>            // Log, LogSetGlobalLevel
@@ -77,6 +78,13 @@ static const char *const CF_NET_MANPAGE_LONG_DESCRIPTION =
     "host. This is due to ACLs and key-pairs and can be made easier in the "
     "future.";
 
+static const Component COMPONENT =
+{
+    .name = "cf-net",
+    .website = CF_WEBSITE,
+    .copyright = CF_COPYRIGHT
+};
+
 static const Description COMMANDS[] =
 {
     {"help",    "Prints general help or per topic",
@@ -156,7 +164,7 @@ static const char *command_strings[] =
 
 // INIT:
 static void CFNetSetDefault(CFNetOptions *opts);
-static void CFNetInit();
+static void CFNetInit(const char *min_tls_version, const char *allow_ciphers);
 static void CFNetOptionsClear(CFNetOptions *opts);
 
 // MAIN LOGIC:
@@ -557,7 +565,7 @@ static int CFNetHelp(const char *topic)
     else
     {
         Writer *w = FileWriter(stdout);
-        WriterWriteHelp(w, "cf-net", OPTIONS, HINTS, COMMANDS, false, false);
+        WriterWriteHelp(w, &COMPONENT, OPTIONS, HINTS, COMMANDS, false, false);
         FileWriterDetach(w);
         DoCleanupAndExit(EXIT_SUCCESS);
     }
