@@ -1,5 +1,5 @@
 /*
-   Copyright 2021 Northern.tech AS
+   Copyright 2024 Northern.tech AS
 
    This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -44,6 +44,7 @@
 */
 
 #include <platform.h>
+#include <getopt.h>
 #include <openssl/err.h>
 
 #include <lastseen.h>
@@ -95,6 +96,13 @@ static const char *const CF_SECRET_MANPAGE_LONG_DESCRIPTION =
     "corresponding private key. Original author: Jon Henrik Bjornstad "
     "<jonhenrik@cfengineers.net>";
 
+static const Component COMPONENT =
+{
+    .name = "cf-secrets",
+    .website = CF_WEBSITE,
+    .copyright = CF_COPYRIGHT
+};
+
 static const struct option OPTIONS[] =
 {
     {"help",        no_argument,        0, 'h'},
@@ -831,7 +839,7 @@ static Seq *LoadPublicKeys(Seq *key_paths)
 static void CFKeyCryptHelp()
 {
     Writer *w = FileWriter(stdout);
-    WriterWriteHelp(w, "cf-secret", OPTIONS, HINTS, COMMANDS, true, true);
+    WriterWriteHelp(w, &COMPONENT, OPTIONS, HINTS, COMMANDS, true, true);
     FileWriterDetach(w);
 }
 
