@@ -1,5 +1,5 @@
 /*
-  Copyright 2022 Northern.tech AS
+  Copyright 2024 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -44,8 +44,7 @@ static void RemoveTimeClass(EvalContext *ctx, const char* tags)
     RlistDestroy(tags_rlist);
 }
 
-static void AddTimeClass(EvalContext *ctx, time_t time, const char* tags)
-{
+StringSet *GetTimeClasses(time_t time) {
     // The first element is the local timezone
     const char* tz_prefix[2] = { "", "GMT_" };
     const char* tz_function[2] = { "localtime_r", "gmtime_r" };
@@ -55,31 +54,29 @@ static void AddTimeClass(EvalContext *ctx, time_t time, const char* tags)
         gmtime_r(&time, &(tz_parsed_time[1]))
     };
 
+    StringSet *classes = StringSetNew();
+
     for (int tz = 0; tz < 2; tz++)
     {
-        char buf[CF_BUFSIZE];
         int day_text_index, quarter, interval_start, interval_end;
 
         if (tz_tm[tz] == NULL)
         {
             Log(LOG_LEVEL_ERR, "Unable to parse passed time. (%s: %s)", tz_function[tz], GetErrorStr());
-            return;
+            return NULL;
         }
 
 /* Lifecycle */
 
-        snprintf(buf, CF_BUFSIZE, "%sLcycle_%d", tz_prefix[tz], ((tz_parsed_time[tz].tm_year + 1900) % 3));
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%sLcycle_%d", tz_prefix[tz], ((tz_parsed_time[tz].tm_year + 1900) % 3));
 
 /* Year */
 
-        snprintf(buf, CF_BUFSIZE, "%sYr%04d", tz_prefix[tz], tz_parsed_time[tz].tm_year + 1900);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%sYr%04d", tz_prefix[tz], tz_parsed_time[tz].tm_year + 1900);
 
 /* Month */
 
-        snprintf(buf, CF_BUFSIZE, "%s%s", tz_prefix[tz], MONTH_TEXT[tz_parsed_time[tz].tm_mon]);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%s%s", tz_prefix[tz], MONTH_TEXT[tz_parsed_time[tz].tm_mon]);
 
 /* Day of week */
 
@@ -88,46 +85,55 @@ static void AddTimeClass(EvalContext *ctx, time_t time, const char* tags)
    ...
    Sunday  is 0 in tm_wday, 6 in DAY_TEXT */
         day_text_index = (tz_parsed_time[tz].tm_wday + 6) % 7;
-        snprintf(buf, CF_BUFSIZE, "%s%s", tz_prefix[tz], DAY_TEXT[day_text_index]);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%s%s", tz_prefix[tz], DAY_TEXT[day_text_index]);
 
 /* Day */
 
-        snprintf(buf, CF_BUFSIZE, "%sDay%d", tz_prefix[tz], tz_parsed_time[tz].tm_mday);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%sDay%d", tz_prefix[tz], tz_parsed_time[tz].tm_mday);
 
 /* Shift */
 
-        snprintf(buf, CF_BUFSIZE, "%s%s", tz_prefix[tz], SHIFT_TEXT[tz_parsed_time[tz].tm_hour / 6]);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%s%s", tz_prefix[tz], SHIFT_TEXT[tz_parsed_time[tz].tm_hour / 6]);
 
 /* Hour */
 
-        snprintf(buf, CF_BUFSIZE, "%sHr%02d", tz_prefix[tz], tz_parsed_time[tz].tm_hour);
-        EvalContextClassPutHard(ctx, buf, tags);
-        snprintf(buf, CF_BUFSIZE, "%sHr%d", tz_prefix[tz], tz_parsed_time[tz].tm_hour);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%sHr%02d", tz_prefix[tz], tz_parsed_time[tz].tm_hour);
+        StringSetAddF(classes, "%sHr%d", tz_prefix[tz], tz_parsed_time[tz].tm_hour);
 
 /* Quarter */
 
         quarter = tz_parsed_time[tz].tm_min / 15 + 1;
 
-        snprintf(buf, CF_BUFSIZE, "%sQ%d", tz_prefix[tz], quarter);
-        EvalContextClassPutHard(ctx, buf, tags);
-        snprintf(buf, CF_BUFSIZE, "%sHr%02d_Q%d", tz_prefix[tz], tz_parsed_time[tz].tm_hour, quarter);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%sQ%d", tz_prefix[tz], quarter);
+        StringSetAddF(classes, "%sHr%02d_Q%d", tz_prefix[tz], tz_parsed_time[tz].tm_hour, quarter);
 
 /* Minute */
 
-        snprintf(buf, CF_BUFSIZE, "%sMin%02d", tz_prefix[tz], tz_parsed_time[tz].tm_min);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%sMin%02d", tz_prefix[tz], tz_parsed_time[tz].tm_min);
 
         interval_start = (tz_parsed_time[tz].tm_min / 5) * 5;
         interval_end = (interval_start + 5) % 60;
 
-        snprintf(buf, CF_BUFSIZE, "%sMin%02d_%02d", tz_prefix[tz], interval_start, interval_end);
-        EvalContextClassPutHard(ctx, buf, tags);
+        StringSetAddF(classes, "%sMin%02d_%02d", tz_prefix[tz], interval_start, interval_end);
     }
+
+    return classes;
+}
+
+static void AddTimeClass(EvalContext *ctx, time_t time, const char* tags)
+{
+    StringSet *time_classes = GetTimeClasses(time);
+    if (time_classes == NULL) {
+        return;
+    }
+
+    StringSetIterator iter = StringSetIteratorInit(time_classes);
+    const char *time_class = NULL;
+    while ((time_class = StringSetIteratorNext(&iter)) != NULL) {
+        EvalContextClassPutHard(ctx, time_class, tags);
+    }
+
+    StringSetDestroy(time_classes);
 }
 
 void UpdateTimeClasses(EvalContext *ctx, time_t t)
