@@ -1,20 +1,19 @@
 /*
-  Copyright 2022 Northern.tech AS
+  Copyright 2024 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; version 3.
+  Licensed under the Apache License, Version 2.0 (the "License");
+  you may not use this file except in compliance with the License.
+  You may obtain a copy of the License at
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+      http://www.apache.org/licenses/LICENSE-2.0
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
 
   To the extent this program is licensed as part of the Enterprise
   versions of CFEngine, the applicable Commercial Open Source License
@@ -35,9 +34,23 @@
 #include <string_lib.h>                                         /* memcchr */
 #include <path.h>
 
-#ifndef __MINGW32__
-#include <glob.h>
-#else
+/* below are includes for the fancy efficient file/data copying on Linux */
+#ifdef __linux__
+
+
+#ifdef HAVE_LINUX_FS_H
+#include <linux/fs.h>  /* FICLONE */
+#endif
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#ifdef HAVE_SYS_SENDFILE_H
+#include <sys/sendfile.h>
+#endif
+#include <sys/stat.h>
+#include <unistd.h>    /* copy_file_range(), lseek() */
+#endif /* __linux__ */
+
+#ifdef __MINGW32__
 #include <windows.h>            /* LockFileEx and friends */
 #endif
 
@@ -330,6 +343,48 @@ NewLineMode FileNewLineMode(ARG_UNUSED const char *file)
 }
 #endif // !__MINGW32__
 
+bool IsWindowsNetworkPath(const char *const path)
+{
+#ifdef _WIN32
+    int off = 0;
+
+    while (path[off] == '\"')
+    {
+        // Bypass quoted strings
+        off += 1;
+    }
+
+    if (IsFileSep(path[off]) && IsFileSep(path[off + 1]))
+    {
+        return true;
+    }
+#else // _WIN32
+    UNUSED(path);
+#endif // _WIN32
+    return false;
+}
+
+bool IsWindowsDiskPath(const char *const path)
+{
+#ifdef _WIN32
+    int off = 0;
+
+    while (path[off] == '\"')
+    {
+        // Bypass quoted strings
+        off += 1;
+    }
+
+    if (isalpha(path[off]) && path[off + 1] == ':' && IsFileSep(path[off + 2]))
+    {
+        return true;
+    }
+#else // _WIN32
+    UNUSED(path);
+#endif // _WIN32
+    return false;
+}
+
 bool IsAbsoluteFileName(const char *f)
 {
     int off = 0;
@@ -340,17 +395,14 @@ bool IsAbsoluteFileName(const char *f)
     {
     }
 
-#ifdef _WIN32
-    if (IsFileSep(f[off]) && IsFileSep(f[off + 1]))
+    if (IsWindowsNetworkPath(f))
     {
         return true;
     }
-
-    if (isalpha(f[off]) && f[off + 1] == ':' && IsFileSep(f[off + 2]))
+    if (IsWindowsDiskPath(f))
     {
         return true;
     }
-#endif
     if (IsFileSep(f[off]))
     {
         return true;
@@ -468,6 +520,109 @@ void switch_symlink_hook();
 #define TEST_SYMLINK_SWITCH_POINT
 #endif
 
+void PathWalk(
+    const char *const path,
+    PathWalkFn callback,
+    void *const data,
+    PathWalkCopyFn copy,
+    PathWalkDestroyFn destroy)
+{
+    assert(path != NULL);
+    assert(callback != NULL);
+
+    Seq *const children = ListDir(path, NULL);
+    if (children == NULL) {
+        Log(
+            LOG_LEVEL_DEBUG,
+            "Failed to list directory '%s'. Subdirectories will not be "
+            "iterated.", path);
+        return;
+    }
+    const size_t n_children = SeqLength(children);
+
+    Seq *const dirnames = SeqNew(1, free);
+    Seq *const filenames = SeqNew(1, free);
+
+    for (size_t i = 0; i < n_children; i++)
+    {
+        char *const child = SeqAt(children, i);
+
+        /* The basename(3) function might potentially mutate the argument.
+         * Thus, we make a copy to pass instead. */
+        char buf[PATH_MAX];
+        const size_t ret = StringCopy(child, buf, PATH_MAX);
+        if (ret >= PATH_MAX) {
+            Log(LOG_LEVEL_ERR,
+                "Failed to copy path: Path too long (%zu >= %d)",
+                ret, PATH_MAX);
+            SeqDestroy(dirnames);
+            SeqDestroy(filenames);
+            return;
+        }
+        const char *const b_name = basename(buf);
+
+        /* We don't iterate the '.' and '..' directory entries as it would cause
+         * infinite recursion. */
+        if (StringEqual(b_name, ".") || StringEqual(b_name, ".."))
+        {
+            continue;
+        }
+
+        // Note that stat(2) follows symbolic links.
+        struct stat sb;
+        if (stat(child, &sb) == 0)
+        {
+            char *const dup = xstrdup(b_name);
+            if (sb.st_mode & S_IFDIR)
+            {
+                SeqAppend(dirnames, dup);
+            }
+            else
+            {
+                SeqAppend(filenames, dup);
+            }
+        }
+        else
+        {
+            Log(LOG_LEVEL_DEBUG,
+                "Failed to stat file '%s': %s",
+                child,
+                GetErrorStr());
+        }
+    }
+
+    SeqDestroy(children);
+    callback(path, dirnames, filenames, data);
+    SeqDestroy(filenames);
+
+    // Recursively walk through subdirectories.
+    const size_t n_dirs = SeqLength(dirnames);
+    for (size_t i = 0; i < n_dirs; i++)
+    {
+        const char *const dir = SeqAt(dirnames, i);
+        if (dir != NULL)
+        {
+            char *const duplicate = (copy != NULL) ? copy(data) : data;
+            if (StringEqual(path, "."))
+            {
+                PathWalk(dir, callback, duplicate, copy, destroy);
+            }
+            else
+            {
+                char *next = Path_JoinAlloc(path, dir);
+                PathWalk(next, callback, duplicate, copy, destroy);
+                free(next);
+            }
+            if (copy != NULL && destroy != NULL)
+            {
+                destroy(duplicate);
+            }
+        }
+    }
+
+    SeqDestroy(dirnames);
+}
+
 Seq *ListDir(const char *dir, const char *extension)
 {
     Dir *dirh = DirOpen(dir);
@@ -939,6 +1094,8 @@ int safe_chdir(const char *path)
  *                       Initially this is false, but will be set to true in
  *                       sub invocations if we follow a link.
  * @param loop_countdown Protection against infinite loop following.
+ * @param true_leaf      Place to store the true leaf name (basename) *initialized
+ *                       to %NULL* or %NULL
  * @return File descriptor pointing to the parent directory of path, or -1 on
  *         error.
  */
@@ -947,8 +1104,11 @@ static int safe_open_true_parent_dir(const char *path,
                                      uid_t link_user,
                                      gid_t link_group,
                                      bool traversed_link,
-                                     int loop_countdown)
+                                     int loop_countdown,
+                                     char **true_leaf)
 {
+    assert((true_leaf == NULL) || (*true_leaf == NULL));
+
     int dirfd = -1;
     int ret = -1;
 
@@ -1022,7 +1182,8 @@ static int safe_open_true_parent_dir(const char *path,
         }
 
         ret = safe_open_true_parent_dir(resolved_link, flags, statbuf.st_uid,
-                                        statbuf.st_gid, true, loop_countdown);
+                                        statbuf.st_gid, true, loop_countdown,
+                                        true_leaf);
 
         free(resolved_link);
         goto cleanup;
@@ -1035,6 +1196,14 @@ static int safe_open_true_parent_dir(const char *path,
 
 cleanup:
     free(parent_dir_alloc);
+
+    /* We only want to set this once, in the deepest recursive call of this
+     * function (see above). */
+    if ((true_leaf != NULL) && (*true_leaf == NULL))
+    {
+        *true_leaf = xstrdup(leaf);
+    }
+
     free(leaf_alloc);
 
     if (dirfd != -1)
@@ -1047,29 +1216,36 @@ cleanup:
 
 /**
  * Implementation of safe_chown.
- * @param path Path to chown.
- * @param owner          Owner to set on path.
- * @param group          Group to set on path.
- * @param flags          Flags to use for fchownat.
- * @param link_user      If we have traversed a link already, which user was it.
- * @param link_group     If we have traversed a link already, which group was it.
- * @param traversed_link Whether we have traversed a link. If this is false the
- *                       two previus arguments are ignored. This is used enforce
- *                       the correct UID/GID combination when following links.
- *                       Initially this is false, but will be set to true in
- *                       sub invocations if we follow a link.
- * @param loop_countdown Protection against infinite loop following.
+ * @param path   Path to chown.
+ * @param owner  Owner to set on path.
+ * @param group  Group to set on path.
+ * @param flags  Flags to use for fchownat.
  */
-int safe_chown_impl(const char *path, uid_t owner, gid_t group, int flags)
+static int safe_chown_impl(const char *path, uid_t owner, gid_t group, int flags)
 {
-    int dirfd = safe_open_true_parent_dir(path, flags, 0, 0, false, SYMLINK_MAX_DEPTH);
+    int dirfd;
+    char *leaf_alloc = NULL;
+    char *leaf;
+    if ((flags & AT_SYMLINK_NOFOLLOW) != 0)
+    {
+        /* SYMLINK_NOFOLLOW set, we can take leaf name from path */
+        dirfd = safe_open_true_parent_dir(path, flags, 0, 0, false, SYMLINK_MAX_DEPTH, NULL);
+        leaf_alloc = xstrdup(path);
+        leaf = basename(leaf_alloc);
+    }
+    else
+    {
+        /* no SYMLINK_NOFOLLOW -> follow symlinks, thus we need the real leaf
+         * name */
+        dirfd = safe_open_true_parent_dir(path, flags, 0, 0, false, SYMLINK_MAX_DEPTH, &leaf_alloc);
+        leaf = leaf_alloc;
+    }
     if (dirfd < 0)
     {
+        free(leaf_alloc);
         return -1;
     }
 
-    char *leaf_alloc = xstrdup(path);
-    char *leaf = basename(leaf_alloc);
 
     // We now know it either isn't a link, or we don't want to follow it if it
     // is. In either case make sure we don't try to follow it.
@@ -1127,21 +1303,20 @@ int safe_chmod(const char *path, mode_t mode)
     int dirfd = -1;
     int ret = -1;
 
-    char *leaf_alloc = xstrdup(path);
-    char *leaf = basename(leaf_alloc);
-    struct stat statbuf;
-    uid_t olduid = 0;
-
-    if ((dirfd = safe_open_true_parent_dir(path, 0, 0, 0, false, SYMLINK_MAX_DEPTH)) == -1)
+    char *leaf_alloc = NULL;
+    if ((dirfd = safe_open_true_parent_dir(path, 0, 0, 0, false, SYMLINK_MAX_DEPTH, &leaf_alloc)) == -1)
     {
         goto cleanup;
     }
 
+    char *leaf = basename(leaf_alloc);
+    struct stat statbuf;
     if ((ret = fstatat(dirfd, leaf, &statbuf, AT_SYMLINK_NOFOLLOW)) == -1)
     {
         goto cleanup;
     }
 
+    uid_t olduid = 0;
     if (S_ISFIFO(statbuf.st_mode) || S_ISSOCK(statbuf.st_mode))
     {
         /* For FIFOs/sockets we cannot resort to the method of opening the file
@@ -1346,20 +1521,30 @@ bool FileSparseWrite(int fd, const void *buf, size_t count,
 }
 
 /**
- * Copy data jumping over areas filled by '\0' greater than blk_size, so
- * files automatically become sparse if possible.
+ * Below is a basic implementation of copying sparse files shoveling data from
+ * sd to dd detecting blocks of zeroes.
  *
- * File descriptors should already be open, the filenames #source and
- * #destination are only for logging purposes.
- *
- * @NOTE Always use FileSparseClose() to close the file descriptor, to avoid
- *       losing data.
+ * On Linux, we try more fancy and efficient approaches than what is done
+ * below if any are supported.
  */
-bool FileSparseCopy(int sd, const char *src_name,
-                    int dd, const char *dst_name,
-                    size_t blk_size,
-                    size_t *total_bytes_written,
-                    bool   *last_write_was_a_hole)
+#ifdef __linux__
+#if HAVE_SENDFILE || HAVE_COPY_FILE_RANGE
+#if HAVE_DECL_SEEK_DATA && HAVE_DECL_FALLOC_FL_PUNCH_HOLE
+#define SMART_FILE_COPY_SYSCALLS_AVAILABLE 1
+#define SMART_SYSCALLS_UNUSED ARG_UNUSED
+#endif  /* HAVE_DECL_SEEK_DATA && HAVE_DECL_FALLOC_FL_PUNCH_HOLE */
+#endif  /* HAVE_SENDFILE || HAVE_COPY_FILE_RANGE */
+#endif  /* __linux__ */
+#ifndef SMART_FILE_COPY_SYSCALLS_AVAILABLE
+#define SMART_FILE_COPY_SYSCALLS_AVAILABLE 0
+#define SMART_SYSCALLS_UNUSED
+#endif
+
+SMART_SYSCALLS_UNUSED static bool FileSparseCopyShoveling(int sd, const char *src_name,
+                                                          int dd, const char *dst_name,
+                                                          size_t blk_size,
+                                                          size_t *total_bytes_written,
+                                                          bool   *last_write_was_a_hole)
 {
     assert(total_bytes_written   != NULL);
     assert(last_write_was_a_hole != NULL);
@@ -1405,6 +1590,188 @@ bool FileSparseCopy(int sd, const char *src_name,
     return retval;
 }
 
+/**
+ * Copy data jumping over areas filled by '\0' greater than blk_size, so
+ * files automatically become sparse if possible.
+ *
+ * File descriptors should already be open, the filenames #source and
+ * #destination are only for logging purposes.
+ *
+ * @NOTE Always use FileSparseClose() to close the file descriptor, to avoid
+ *       losing data.
+ */
+bool FileSparseCopy(int sd, const char *src_name,
+                    int dd, const char *dst_name,
+                    SMART_SYSCALLS_UNUSED size_t blk_size,
+                    size_t *total_bytes_written,
+                    bool *last_write_was_a_hole)
+{
+    assert(total_bytes_written != NULL);
+    assert(last_write_was_a_hole != NULL);
+
+#if !SMART_FILE_COPY_SYSCALLS_AVAILABLE
+    return FileSparseCopyShoveling(sd, src_name, dd, dst_name, blk_size,
+                                   total_bytes_written, last_write_was_a_hole);
+#else
+    /* We rely on the errno value below so make sure it's not spoofed by an
+     * error from outside. */
+    errno = 0;
+
+    size_t input_size;
+    struct stat in_sb;
+    if (fstat(sd, &in_sb) == 0)
+    {
+        input_size = in_sb.st_size;
+    }
+    else
+    {
+        Log(LOG_LEVEL_ERR, "Failed to stat() '%s': %m", src_name);
+        *total_bytes_written = 0;
+        return false;
+    }
+    bool same_dev = false;
+    struct stat out_sb;
+    if (fstat(dd, &out_sb) == 0)
+    {
+        same_dev = in_sb.st_dev == out_sb.st_dev;
+    }
+    else
+    {
+        Log(LOG_LEVEL_ERR, "Failed to stat() '%s': %m", dst_name);
+        *total_bytes_written = 0;
+        return false;
+    }
+#if HAVE_DECL_FICLONE
+    if (same_dev)
+    {
+        /* If they are on the same device, first try to use FICLONE
+         * (man:ioctl_ficlone(2)) if available because nothing can beat that.
+         * However, it can easily fail if the FS doesn't support it or if we are
+         * making a copy accross multiple file systems.
+         */
+        if (ioctl(dd, FICLONE, sd) == 0)
+        {
+            Log(LOG_LEVEL_VERBOSE, "'%s' copied successfully to '%s'", src_name, dst_name);
+            *total_bytes_written = input_size;
+            *last_write_was_a_hole = false;
+            return true;
+        }
+        else
+        {
+            Log(LOG_LEVEL_DEBUG, "Failed to use FICLONE to copy '%s' to '%s': %m", src_name, dst_name);
+            /* let's try other things */
+            errno = 0;
+        }
+    }
+#endif  /* HAVE_DECL_FICLONE */
+    if (ftruncate(dd, input_size) != 0)
+    {
+        Log(LOG_LEVEL_ERR, "Failed to preset size for '%s': %m", dst_name);
+        *total_bytes_written = 0;
+        return false;
+    }
+    if (in_sb.st_blocks == 0)
+    {
+        /* Nothing to copy. */
+        Log(LOG_LEVEL_VERBOSE, "'%s' copied successfully to '%s'", src_name, dst_name);
+        *total_bytes_written = input_size;
+        *last_write_was_a_hole = false;
+        return true;
+    }
+
+    /* man:inode(7) says about st_blocks:
+     * This field indicates the number of blocks allocated to the file, 512-byte
+     * units, (This may be smaller than st_size/512 when the file has holes.)
+     * The POSIX.1 standard notes that the unit for the st_blocks member of the stat
+     * structure is not defined by the standard.  On many implementations it is 512
+     * bytes; on a few systems, a different unit is used, such as 1024. Furthermore,
+     * the unit may differ on a per-filesystem basis.
+     * So using 512 is the best we can do although it might not be perfect.
+     */
+    if (fallocate(dd, FALLOC_FL_KEEP_SIZE|FALLOC_FL_PUNCH_HOLE, 0, in_sb.st_blocks * 512) != 0)
+    {
+        Log(LOG_LEVEL_ERR, "Failed to pre-allocate space for '%s': %m", dst_name);
+        *total_bytes_written = 0;
+        return false;
+    }
+
+    off_t in_pos = 0;
+    off_t out_pos = 0;
+    bool done = false;
+    int error = 0;
+    while (!done)
+    {
+        error = 0;
+        in_pos = lseek(sd, in_pos, SEEK_DATA);
+        if (in_pos == -1)
+        {
+            if (errno == ENXIO)
+            {
+                /* ENXIO means we are either seeking past the end of file or
+                   that we seek data and there is only a hole till the end of
+                   the file. IOW, we are done. */
+                done = true;
+                break;
+            }
+            else
+            {
+                error = errno;
+                Log(LOG_LEVEL_ERR, "Failed to seek to next data in '%s'", src_name);
+                break;
+            }
+        }
+        if (in_pos != out_pos)
+        {
+            out_pos = lseek(dd, in_pos - out_pos, SEEK_CUR);
+            if (out_pos == -1) {
+                error = errno;
+                Log(LOG_LEVEL_ERR, "Failed to advance descriptor in '%s'", dst_name);
+                break;
+            }
+        }
+        off_t next_hole = lseek(sd, in_pos, SEEK_HOLE);
+        if (next_hole == -1)
+        {
+            error = errno;
+            Log(LOG_LEVEL_ERR, "Failed to find next hole in '%s'", src_name);
+            break;
+        }
+        if (lseek(sd, in_pos, SEEK_SET) == -1)
+        {
+            error = errno;
+            Log(LOG_LEVEL_ERR, "Failed to seek back to data in '%s'", src_name);
+            break;
+        }
+        ssize_t n_copied;
+#ifdef HAVE_COPY_FILE_RANGE
+        /* copy_file_range() only works on the same device (file system), but
+         * unlike sendfile() it can actually create reflinks instead of copying
+         * the data. */
+        if (same_dev)
+        {
+            n_copied = copy_file_range(sd, NULL, dd, NULL, next_hole - in_pos, 0);
+            error = errno;
+        }
+        else
+#endif /* HAVE_COPY_FILE_RANGE */
+        {
+            n_copied = sendfile(dd, sd, NULL, next_hole - in_pos);
+            error = errno;
+        }
+        if (n_copied > 0)
+        {
+            in_pos += n_copied;
+            out_pos += n_copied;
+            *total_bytes_written += n_copied;
+        }
+        done = ((n_copied <= 0) || ((size_t) in_pos == input_size));
+    }
+    *last_write_was_a_hole = false;
+    return (error == 0);
+#endif  /* SMART_FILE_COPY_SYSCALLS_AVAILABLE */
+}
+#undef SMART_FILE_COPY_SYSCALLS_AVAILABLE
+
 /**
  * Always close a written sparse file using this function, else truncation
  * might occur if the last part was a hole.
@@ -1425,9 +1792,20 @@ bool FileSparseCopy(int sd, const char *src_name,
  */
 bool FileSparseClose(int fd, const char *filename,
                      bool do_sync,
-                     size_t total_bytes_written,
-                     bool last_write_was_hole)
+                     SMART_SYSCALLS_UNUSED size_t total_bytes_written,
+                     SMART_SYSCALLS_UNUSED bool last_write_was_hole)
 {
+#if SMART_FILE_COPY_SYSCALLS_AVAILABLE
+    /* Not much to do here with smart syscalls (see above). */
+    bool success = true;
+    if (do_sync && (fsync(fd) != 0))
+    {
+        Log(LOG_LEVEL_WARNING, "Could not sync to disk file '%s': %m)", filename);
+        success = false;
+    }
+    close(fd);
+    return success;
+#else
     if (last_write_was_hole)
     {
         ssize_t ret1 = FullWrite(fd, "", 1);
@@ -1471,7 +1849,9 @@ bool FileSparseClose(int fd, const char *filename,
     }
 
     return true;
+#endif /* SMART_FILE_COPY_SYSCALLS_AVAILABLE */
 }
+#undef SMART_FILE_COPY_SYSCALLS_AVAILABLE
 
 ssize_t CfReadLine(char **buff, size_t *size, FILE *fp)
 {
@@ -1530,47 +1910,6 @@ ssize_t CfReadLines(char **buff, size_t *size, FILE *fp, Seq *lines)
     return appended;
 }
 
-StringSet* GlobFileList(const char *pattern)
-{
-    StringSet *set = StringSetNew();
-    glob_t globbuf;
-    int globflags = 0; // TODO: maybe add GLOB_BRACE later
-
-    const char* r_candidates[] = { "*", "*/*", "*/*/*", "*/*/*/*", "*/*/*/*/*", "*/*/*/*/*/*" };
-    bool starstar = ( strstr(pattern, "**") != NULL );
-    const char** candidates   = starstar ? r_candidates : NULL;
-    const int candidate_count = starstar ? 6 : 1;
-
-    for (int pi = 0; pi < candidate_count; pi++)
-    {
-        char *expanded = starstar ?
-            SearchAndReplace(pattern, "**", candidates[pi]) :
-            xstrdup(pattern);
-
-#ifdef _WIN32
-        if (strchr(expanded, '\\'))
-        {
-            Log(LOG_LEVEL_VERBOSE, "Found backslash escape character in glob pattern '%s'. "
-                "Was forward slash intended?", expanded);
-        }
-#endif
-
-        if (glob(expanded, globflags, NULL, &globbuf) == 0)
-        {
-            for (size_t i = 0; i < globbuf.gl_pathc; i++)
-            {
-                StringSetAdd(set, xstrdup(globbuf.gl_pathv[i]));
-            }
-
-            globfree(&globbuf);
-        }
-
-        free(expanded);
-    }
-
-    return set;
-}
-
 /*******************************************************************/
 
 const char* GetRelocatedProcdirRoot()
