@@ -1,20 +1,19 @@
 /*
-  Copyright 2022 Northern.tech AS
+  Copyright 2024 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; version 3.
+  Licensed under the Apache License, Version 2.0 (the "License");
+  you may not use this file except in compliance with the License.
+  You may obtain a copy of the License at
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+      http://www.apache.org/licenses/LICENSE-2.0
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
 
   To the extent this program is licensed as part of the Enterprise
   versions of CFEngine, the applicable Commercial Open Source License
@@ -27,6 +26,7 @@
 
 #include <writer.h>
 #include <inttypes.h> // int64_t
+#include <assert.h>
 
 /**
   @brief JSON data-structure.
@@ -111,6 +111,7 @@ typedef enum
     JSON_PARSE_ERROR_OBJECT_OPEN_LVAL,
 
     JSON_PARSE_ERROR_INVALID_START,
+    JSON_PARSE_ERROR_INVALID_END,
     JSON_PARSE_ERROR_NO_LIBYAML,
     JSON_PARSE_ERROR_LIBYAML_FAILURE,
     JSON_PARSE_ERROR_NO_SUCH_FILE,
@@ -156,9 +157,52 @@ char *Json5EscapeData(Slice unescaped_data);
 //////////////////////////////////////////////////////////////////////////////
 
 JsonElement *JsonCopy(const JsonElement *json);
+
+/**
+ * @brief compare two JsonElement instances
+ *
+ * @param a JsonElement
+ * @param b JsonElement
+ * @return int like strcmp(), 0 is equal, non-zero is not equal
+ */
 int JsonCompare(const JsonElement *a, const JsonElement *b);
+
 JsonElement *JsonMerge(const JsonElement *a, const JsonElement *b);
 
+/**
+ * @brief Recursively merge one object into another
+ *
+ * @param base Object to merge data into
+ * @param extra Object to merge data from
+ * @return Pointer to base object (convenient for nested function calls)
+ *
+ * @note The function should not return NULL
+ * @warning Side effects: The base object is modified in place. If this is not
+ *          the intention, consider using JsonObjectMergeDeep
+ */
+JsonElement *JsonObjectMergeDeepInplace(JsonElement *base, const JsonElement *extra);
+
+/**
+ * @brief Recursively merge two objects into a new object
+ *
+ * @param base Object to copy and merge data into
+ * @param extra Object to merge data from
+ * @return Pointer to merged object
+ *
+ * @note The function should not return NULL
+ * @warning Performance penalty: A new object is created. If this is not the
+ *          intension, consider using JsonObjectMergeDeepInplace
+ */
+static inline JsonElement *JsonObjectMergeDeep(const JsonElement *const base, const JsonElement *const extra)
+{
+    assert(base != NULL);
+    /* This could do a little better job by constructing the JsonElement while
+     * merging base and extra. Not sure if it's worth the extra complexity,
+     * though. */
+    JsonElement *const copy = JsonCopy(base);
+    return JsonObjectMergeDeepInplace(copy, extra);
+}
+
 /**
   @brief Destroy a JSON element
   @param element [in] The JSON element to destroy.
@@ -193,6 +237,7 @@ const char *JsonGetPropertyAsString(const JsonElement *element);
 // JSON Primitives
 //////////////////////////////////////////////////////////////////////////////
 
+const char *JsonContainerTypeToString(JsonContainerType type);
 const char *JsonPrimitiveTypeToString(JsonPrimitiveType type);
 JsonPrimitiveType JsonGetPrimitiveType(const JsonElement *primitive);
 const char *JsonPrimitiveGetAsString(const JsonElement *primitive);
@@ -544,6 +589,18 @@ typedef JsonElement *JsonLookup(void *ctx, const char **data);
   */
 JsonParseError JsonParse(const char **data, JsonElement **json_out);
 
+/**
+  @brief Parse the whole string to create a JsonElement
+  @param data [in] Pointer to the string to parse
+  @param json_out Resulting JSON object
+  @note In contrast to JsonParse(), this function will return
+        JSON_PARSE_ERROR_INVALID_END if there are trailing non-whitespace
+        characters after termination of the JsonElement in the remainder of
+        the string.
+  @returns See JsonParseError and JsonParseErrorToString
+  */
+JsonParseError JsonParseAll(const char **data, JsonElement **json_out);
+
 /**
   @brief Parse a string to create a JsonElement
   @param lookup_data [in] Evaluation context for variable lookups
