@@ -1,20 +1,19 @@
 /*
-  Copyright 2022 Northern.tech AS
+  Copyright 2024 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; version 3.
+  Licensed under the Apache License, Version 2.0 (the "License");
+  you may not use this file except in compliance with the License.
+  You may obtain a copy of the License at
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+      http://www.apache.org/licenses/LICENSE-2.0
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
 
   To the extent this program is licensed as part of the Enterprise
   versions of CFEngine, the applicable Commercial Open Source License
@@ -1782,3 +1781,25 @@ bool StringMatchesOption(
     }
     return StringEqualN_IgnoreCase(supplied, longopt, length);
 }
+
+ssize_t StringFind(
+    const char *const str,
+    const char *const sub,
+    const size_t from,
+    const size_t to)
+{
+    assert(str != NULL);
+    assert(sub != NULL);
+
+    const size_t strl = strlen(str);
+    const size_t subl = strlen(sub);
+
+    for (size_t i = from; i < MIN(strl, to); i++)
+    {
+        if (strncmp(str + i, sub, subl) == 0)
+        {
+            return i;
+        }
+    }
+    return -1;
+}
