@@ -1,4 +1,5 @@
 #include <test.h>
+#include <logging.h>
 
 #include <json.h>
 #include <string_lib.h>
@@ -569,6 +570,25 @@ static void test_copy_compare(void)
     JsonDestroy(copy);
 }
 
+static void test_compare_container_type_mismatch(void)
+{
+    JsonElement *object_a = JsonObjectCreate(1);
+    JsonElement *object_b = JsonObjectCreate(1);
+
+    JsonElement *child = JsonObjectCreate(1);
+    JsonObjectAppendObject(object_a, "key", child);
+
+    JsonElement *array = JsonArrayCreate(1);
+    JsonArrayAppendString(array, "first");
+    JsonArrayAppendString(array, "second");
+    JsonObjectAppendArray(object_b, "key", array);
+
+    assert_true(JsonCompare(object_a, object_b) != 0);
+
+    JsonDestroy(object_a);
+    JsonDestroy(object_b);
+}
+
 static void test_select(void)
 {
     const char *data = OBJECT_ARRAY;
@@ -1294,6 +1314,91 @@ static void test_parse_array_extra_closing(void)
     JsonDestroy(json);
 }
 
+static void test_parse_all(void)
+{
+    const char *data = "\"\"a";
+    JsonElement *json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "\"\""; // Good
+    json = NULL;
+    assert_int_equal(JSON_PARSE_OK, JsonParseAll(&data, &json));
+    assert_true(json != NULL);
+    JsonDestroy(json);
+
+    data = "{}b";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "{}"; // Good
+    json = NULL;
+    assert_int_equal(JSON_PARSE_OK, JsonParseAll(&data, &json));
+    assert_true(json != NULL);
+    JsonDestroy(json);
+
+    data = "[]c";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "{}}";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "{\"d\": \"e\"}}";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "[]]";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "[[]]]";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "[[[]]]]";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "  []]";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "\"some\": [ \"json\" ] }";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "{ \"some\": [ \"json\" ] } [";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "[\"some\", \"json\"]!";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "    [\"some\", \"json\"]a";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+
+    data = "[\"some\", \"json\"] {\"foo\": \"var\"}   ";
+    json = NULL;
+    assert_int_equal(JSON_PARSE_ERROR_INVALID_END, JsonParseAll(&data, &json));
+    assert_true(json == NULL);
+}
+
 static void test_parse_array_diverse(void)
 {
     {
@@ -1823,6 +1928,442 @@ static void test_json_null_not_null(void)
     JsonDestroy(json);
 }
 
+static bool check_json_object_merge_deep(const char *base_raw, const char *extra_raw, const char *expected_raw)
+{
+    LogSetGlobalLevel(LOG_LEVEL_DEBUG);
+    bool pass = true;
+    JsonElement *actual = NULL;
+    JsonElement *base = NULL;
+    JsonElement *extra = NULL;
+    JsonElement *expected = NULL;
+    char *expected_string = NULL;
+    char *actual_string = NULL;
+
+    if (JsonParse(&base_raw, &base) != JSON_PARSE_OK)
+    {
+        printf("error: could not parse base: %s\n", base_raw);
+        pass = false;
+        goto finish;
+    }
+
+    if (JsonParse(&extra_raw, &extra) != JSON_PARSE_OK)
+    {
+        printf("error: could not parse extra: %s\n", extra_raw);
+        pass = false;
+        goto finish;
+    }
+
+    if (JsonParse(&expected_raw, &expected) != JSON_PARSE_OK) {
+        printf("error: could not parse expected: %s\n", expected_raw);
+        pass = false;
+        goto finish;
+    }
+
+    expected_string = JsonToString(expected);
+
+    actual = JsonObjectMergeDeep(base, extra);
+    if (actual == NULL || actual == base)
+    {
+        pass = false;
+        goto finish;
+    }
+
+    actual_string = JsonToString(actual);
+
+    if (JsonCompare(actual, expected) != 0)
+    {
+        pass = false;
+        goto finish;
+    }
+
+    DESTROY_AND_NULL(JsonDestroy,actual);
+    actual = JsonObjectMergeDeepInplace(base, extra);
+
+    // JsonObjectMergeDeepInplace() returns the first parameter (base) so actual should point to base
+    if (actual != base)
+    {
+        pass = false;
+        goto finish;
+    }
+
+    if (JsonCompare(actual, expected))
+    {
+        pass = false;
+        goto finish;
+    }
+
+    finish:
+        if (actual_string)
+        {
+            if (pass == false)
+            {
+                printf("merged json  : %s\n", actual_string);
+            }
+            free(actual_string);
+        }
+        if (expected_string)
+        {
+            if (pass == false)
+            {
+                printf("expected json: %s\n", expected_string);
+            }
+            free(expected_string);
+        }
+        // in second step of JsonObjectMergeDeepInplace() actual should point to base
+        if (actual != base)
+        {
+            JsonDestroy(actual);
+        }
+        JsonDestroy(base);
+        JsonDestroy(extra);
+        JsonDestroy(expected);
+
+    return pass;
+}
+
+static void test_json_object_merge_deep()
+{
+    /* This unit test tests both JsonMergeObjectDeep and
+     * JsonMergeObjectDeepInplace */
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{}",
+        // extra
+        "{}",
+        // expected
+        "{}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{}",
+        // extra
+        "{"
+        "  \"variables\": {}"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {}"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {}"
+        "}",
+        // extra
+        "{}",
+        // expected
+        "{"
+        "  \"variables\": {}"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {}"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {}"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {}"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {}"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_one\": {}"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_one\": {}"
+        "  }"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_two\": {"
+        "      \"value\": [ \"/tmp/virus\" ],"
+        "      \"tags\": [ \"foo\", \"bar\", \"bas\" ]"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_two\": {"
+        "      \"value\": [ \"/tmp/malicious\" ],"
+        "      \"comment\": [ \"Delete dangerous files!\" ]"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_two\": {"
+        "      \"comment\": [ \"Delete dangerous files!\" ],"
+        "      \"tags\": [ \"foo\", \"bar\", \"bas\" ],"
+        "      \"value\": [ \"/tmp/virus\", \"/tmp/malicious\" ]"
+        "    }"
+        "  }"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"classes\": {"
+        "    \"cfbs_delete_files_enable\": {"
+        "      \"class_expressions\": ["
+        "        \"linux.redhat::\","
+        "        \"cfengine|linux::\""
+        "      ],"
+        "      \"tags\": [ \"bogus\", \"doofus\", \"bonkers\" ]"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_three\": {"
+        "      \"value\": [ \"/tmp/malicious\" ],"
+        "      \"comment\": [ \"Delete dangerous files!\" ]"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_three\": {"
+        "      \"comment\": [ \"Delete dangerous files!\" ],"
+        "      \"value\": [ \"/tmp/malicious\" ]"
+        "    }"
+        "  },"
+        "  \"classes\": {"
+        "    \"cfbs_delete_files_enable\": {"
+        "      \"class_expressions\": ["
+        "        \"linux.redhat::\","
+        "        \"cfengine|linux::\""
+        "      ],"
+        "      \"tags\": [ \"bogus\", \"doofus\", \"bonkers\" ]"
+        "    }"
+        "  }"
+        "}"
+    ));
+
+    // NOTE: this assert_false() test is the same as the next one with the exception of value:/tmp/foobad to check deep compare
+    assert_false(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_bad\": {"
+        "      \"value\": [ \"/tmp/foo\", \"/tmp/bar\" ],"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_bad\": {"
+        "      \"value\": [ \"/tmp/bar\", \"/tmp/baz\" ],"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_bad\": {"
+        "      \"value\": [ \"/tmp/foobad\", \"/tmp/bar\", \"/tmp/bar\", \"/tmp/baz\" ],"
+        "    }"
+        "  }"
+        "}"
+    ));
+
+    // this second negative test checks that a mismatch of a second child of an object is found
+    // note the "badchild" in the expectation
+    // there was a bug as I was working here where only the number of children mattered if the first matched, any more than that were not compared
+    assert_false(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables_bad_two\": {"
+        "    \"cfbs:delete_files.filenames_bad_two\": {"
+        "      \"value\": [ \"/tmp/foo\", \"/tmp/bar\" ],"
+        "      \"badchild\": { \"one\": \"uno\", \"two\": \"due\" },"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables_bad_two\": {"
+        "    \"cfbs:delete_files.filenames_bad_two\": {"
+        "      \"value\": [ \"/tmp/bar\", \"/tmp/baz\" ],"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables_bad_two\": {"
+        "    \"cfbs:delete_files.filenames_bad_two\": {"
+        "      \"value\": [ \"/tmp/foo\", \"/tmp/bar\", \"/tmp/bar\", \"/tmp/baz\" ],"
+        "      \"badchild\": { \"one\": \"uno\", \"two\": \"dva\" },"
+        "    }"
+        "  }"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables_four\": {"
+        "    \"cfbs:delete_files.filenames_four\": {"
+        "      \"value\": [ \"/tmp/foo\", \"/tmp/bar\" ],"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables_four\": {"
+        "    \"cfbs:delete_files.filenames_four\": {"
+        "      \"value\": [ \"/tmp/bar\", \"/tmp/baz\" ],"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables_four\": {"
+        "    \"cfbs:delete_files.filenames_four\": {"
+        "      \"value\": [ \"/tmp/foo\", \"/tmp/bar\", \"/tmp/bar\", \"/tmp/baz\" ],"
+        "    }"
+        "  }"
+        "}"
+    ));
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_five\": {"
+        "      \"value\": [],"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_five\": {"
+        "      \"value\": {},"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_five\": {"
+        "      \"value\": {},"
+        "    }"
+        "  }"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_six\": {"
+        "      \"value\": {},"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_six\": {"
+        "      \"value\": [],"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_six\": {"
+        "      \"value\": [],"
+        "    }"
+        "  }"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_seven\": {"
+        "      \"value\": [],"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_seven\": {"
+        "      \"value\": 123,"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_seven\": {"
+        "      \"value\": 123,"
+        "    }"
+        "  }"
+        "}"
+    ));
+
+    assert_true(check_json_object_merge_deep(
+        // base
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_eight\": {"
+        "      \"value\": 123,"
+        "    }"
+        "  }"
+        "}",
+        // extra
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_eight\": {"
+        "      \"value\": {},"
+        "    }"
+        "  }"
+        "}",
+        // expected
+        "{"
+        "  \"variables\": {"
+        "    \"cfbs:delete_files.filenames_eight\": {"
+        "      \"value\": {},"
+        "    }"
+        "  }"
+        "}"
+    ));
+}
+
+
 int main()
 {
     PRINT_TEST_BANNER();
@@ -1848,6 +2389,7 @@ int main()
         unit_test(test_parse_array_diverse),
         unit_test(test_parse_array_double_and_trailing_commas),
         unit_test(test_parse_array_extra_closing),
+        unit_test(test_parse_all),
         unit_test(test_parse_array_garbage),
         unit_test(test_parse_array_nested_garbage),
         unit_test(test_parse_array_object),
@@ -1891,6 +2433,8 @@ int main()
         unit_test(test_string_escape),
         unit_test(test_string_escape_json5),
         unit_test(test_json_null_not_null),
+        unit_test(test_json_object_merge_deep),
+        unit_test(test_compare_container_type_mismatch),
     };
 
     return run_tests(tests);
