@@ -31,6 +31,23 @@ static void test_length(void)
     RlistDestroy(list);
 }
 
+static void test_equality(void)
+{
+    Rlist *list1 = RlistFromSplitString("a,b,c", ',');
+    Rlist *list2 = RlistFromSplitString("a,b,c", ',');
+    Rlist *list3 = RlistFromSplitString("z,b,c", ',');
+    Rlist *list4 = RlistFromSplitString("a,b,c,d", ',');
+
+    assert_true(RlistEqual(list1, list2));
+    assert_false(RlistEqual(list1, list3));
+    assert_false(RlistEqual(list1, list4));
+
+    RlistDestroy(list1);
+    RlistDestroy(list2);
+    RlistDestroy(list3);
+    RlistDestroy(list4);
+}
+
 static void test_prepend_scalar_idempotent(void)
 {
     Rlist *list = NULL;
@@ -713,6 +730,36 @@ static void test_regex_split_overlapping_delimiters()
     RlistDestroy(list);
 }
 
+static void test_rval_write()
+{
+    Rval rval = { "\"Hello World!\"", RVAL_TYPE_SCALAR };
+    Writer *writer = StringWriter();
+    RvalWrite(writer, rval);
+    const char *actual = StringWriterData(writer);
+    assert_string_equal(actual, "\\\"Hello World!\\\"");
+    WriterClose(writer);
+}
+
+static void test_rval_write_quoted()
+{
+    Rval rval = { "\"Hello World!\"", RVAL_TYPE_SCALAR };
+    Writer *writer = StringWriter();
+    RvalWriteQuoted(writer, rval);
+    const char *actual = StringWriterData(writer);
+    assert_string_equal(actual, "\"\\\"Hello World!\\\"\"");
+    WriterClose(writer);
+}
+
+static void test_rval_write_raw()
+{
+    Rval rval = { "\"Hello World!\"", RVAL_TYPE_SCALAR };
+    Writer *writer = StringWriter();
+    RvalWriteRaw(writer, rval);
+    const char *actual = StringWriterData(writer);
+    assert_string_equal(actual, "\"Hello World!\"");
+    WriterClose(writer);
+}
+
 int main()
 {
     PRINT_TEST_BANNER();
@@ -720,6 +767,7 @@ int main()
     {
         unit_test(test_prepend_scalar_idempotent),
         unit_test(test_length),
+        unit_test(test_equality),
         unit_test(test_copy),
         unit_test(test_rval_to_scalar),
         unit_test(test_rval_to_scalar2),
@@ -744,7 +792,10 @@ int main()
         unit_test(test_regex_split_no_match),
         unit_test(test_regex_split_adjacent_separators),
         unit_test(test_regex_split_real_regex),
-        unit_test(test_regex_split_overlapping_delimiters)
+        unit_test(test_regex_split_overlapping_delimiters),
+        unit_test(test_rval_write),
+        unit_test(test_rval_write_quoted),
+        unit_test(test_rval_write_raw),
     };
 
     return run_tests(tests);
